@@ -180,10 +180,8 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 				    const gchar *username,
 				    const gchar *password)
 {
-        gchar *ora_tnsname;
-	gchar *ora_username;
-	gchar *ora_password;
-	gint result;
+        const gchar *tnsname;
+	gint  result;
 
         GdaOracleProvider *ora_prv = (GdaOracleProvider *) provider;
 	GdaOracleConnectionData *priv_data;
@@ -191,8 +189,17 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 	g_return_val_if_fail (GDA_IS_ORACLE_PROVIDER (ora_prv), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 
+	/* check we have a TNS name to connect to */
+
+	if ((tnsname = gda_quark_list_find (params, "TNSNAME")) == NULL) {
+		gda_connection_add_error_string (cnc, 
+		        _("No TNS name supplied"));
+		return FALSE;
+	}
+
 	priv_data = g_new0 (GdaOracleConnectionData, 1);
 
+        g_assert (priv_data != NULL);
         /* initialize Oracle */
 	result = OCIInitialize ((ub4) OCI_DEFAULT,
 				(dvoid *) 0,
@@ -200,12 +207,12 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 				(dvoid * (*)(dvoid *, dvoid *, size_t)) 0,
 				(void (*)(dvoid *, dvoid *)) 0);
 
-	if (result != OCI_SUCCESS) { 
+	if (result != OCI_SUCCESS) {
 		gda_connection_add_error_string (cnc, 
 			_("Could not initialize Oracle"));
 		return FALSE;
 	}
-
+	
 	/* initialize the Oracle environment */
 	result = OCIEnvInit ((OCIEnv **) & priv_data->henv, 
 				(ub4) OCI_DEFAULT, 
@@ -217,7 +224,6 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-
 	/* create the service context */
 	result = OCIHandleAlloc ((dvoid *) priv_data->henv,
 				(dvoid **) &priv_data->hservice,
@@ -268,26 +274,26 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 	}
 
 	/* if the username isn't provided, try to find it in the DSN */
-	if (strlen(username) > 0)
-		ora_username = username;
-	else
-		ora_username = gda_quark_list_find (params, "USER");
 
-	/* if the username isn't provided, try to find it in the DSN */
-	if (strlen(password) > 0)
-		ora_password = password;
-	else
-		ora_password = gda_quark_list_find (params, "PASSWORD");
+	if (username == NULL || *username == '\0') {
+		username = gda_quark_list_find (params, "USER");
+		if (username == NULL)
+			username="";
+	}
 
+	/* if the password isn't provided, try to find it in the DSN */
+	if (password == NULL || *password == '\0') {
+		password = gda_quark_list_find (params, "PASSWORD");
+		if (password == NULL)
+			password="";
+	}
 
-	ora_tnsname = gda_quark_list_find (params, "TNSNAME");
-        g_assert (priv_data != NULL);
 
         /* attach to Oracle server */
 	result = OCIServerAttach (priv_data->hserver,
 				priv_data->herr,
-				(text *) ora_tnsname,
-				(ub4) strlen (ora_tnsname),
+				(text *) tnsname,
+				(ub4) strlen (tnsname),
 				OCI_DEFAULT);
 	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
 			_("Could not attach to the Oracle server"))) {
@@ -317,8 +323,8 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 	/* set the username attribute */
 	result = OCIAttrSet ((dvoid *) priv_data->hsession, 
 				(ub4) OCI_HTYPE_SESSION, 
-				(dvoid *) ora_username,
-				(ub4) strlen (ora_username), 
+				(dvoid *) username,
+				(ub4) strlen (username),
 				(ub4) OCI_ATTR_USERNAME,
 				priv_data->herr);
 	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
@@ -333,8 +339,8 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 	/* set the password attribute */
 	result = OCIAttrSet ((dvoid *) priv_data->hsession, 
 				(ub4) OCI_HTYPE_SESSION, 
-				(dvoid *) ora_password,
-				(ub4) strlen (ora_password), 
+				(dvoid *) password,
+				(ub4) strlen (password), 
 				(ub4) OCI_ATTR_PASSWORD,
 				priv_data->herr);
 	if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
@@ -380,9 +386,11 @@ gda_oracle_provider_open_connection (GdaServerProvider *provider,
 		return FALSE;
 	}
 
-	priv_data->schema = g_ascii_strup(ora_username, -1);
+	priv_data->schema = g_ascii_strup(username, -1);
 	priv_data->tables = NULL;
 	priv_data->views = NULL;
+	priv_data->aggregates = NULL;
+	priv_data->procedures = NULL;
 	
 	/* attach the oracle connection data to the gda connection object */
 	g_object_set_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE, priv_data);
@@ -412,7 +420,7 @@ gda_oracle_provider_close_connection (GdaServerProvider *provider, GdaConnection
 				priv_data->hsession,
 				OCI_DEFAULT))) {
 		gda_connection_add_error (cnc, 
-			gda_oracle_make_error (priv_data->herr, OCI_HTYPE_ERROR));
+			gda_oracle_make_error (priv_data->herr, OCI_HTYPE_ERROR, __FILE__, __LINE__));
 		return FALSE;
 	}
 
@@ -428,12 +436,16 @@ gda_oracle_provider_close_connection (GdaServerProvider *provider, GdaConnection
 	if (priv_data->henv)
 		OCIHandleFree ((dvoid *) priv_data->henv, OCI_HTYPE_ENV);
 	if (priv_data->schema)
-	    g_free(priv_data->schema);
+		g_free(priv_data->schema);
 	if (priv_data->tables)
-	    g_tree_destroy(priv_data->tables);
+		g_tree_destroy(priv_data->tables);
 	if (priv_data->views)
-	    g_tree_destroy(priv_data->views);
-							
+		g_tree_destroy(priv_data->views);
+	if (priv_data->aggregates)
+		g_tree_destroy(priv_data->aggregates);
+	if (priv_data->procedures)
+		g_tree_destroy(priv_data->procedures);
+
 	g_free (priv_data);
 	g_object_set_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE, NULL);
 
@@ -828,7 +840,7 @@ gda_oracle_provider_begin_transaction (GdaServerProvider *provider,
 	GdaOracleConnectionData *priv_data;
 	GdaOracleProvider *ora_prv = (GdaOracleProvider *) provider;
 	GdaOracleTransaction *ora_xaction;
-	gchar *xaction_name;
+	const gchar *xaction_name;
 	gint result;
 
 	g_return_val_if_fail (GDA_IS_ORACLE_PROVIDER (ora_prv), FALSE);
@@ -1038,80 +1050,124 @@ gda_oracle_provider_supports (GdaServerProvider *provider,
 	return FALSE;
 }
 
-static void
-add_string_row (GdaDataModelArray *recset, const gchar *str)
+typedef struct
 {
-	GdaValue *value;
-	GList list;
+	const gchar *name;
+	const gchar *uid;
+	const gchar *args;
+} gda_oracle_aggregate_t;
 
-	g_return_if_fail (GDA_IS_DATA_MODEL_ARRAY (recset));
+static gda_oracle_aggregate_t aggregate_tab[] =
+{
+	{ "AVG",             "SYS_AGG01", "NUMBER"        },
+	{ "CORR",            "SYS_AGG02", "NUMBER NUMBER" },
+	{ "COUNT",           "SYS_AGG03", "-"             },
+	{ "COVAR_POP",       "SYS_AGG04", "NUMBER NUMBER" },
+	{ "COVAR_SAMP",      "SYS_AGG05", "NUMBER NUMBER" },
+	{ "CUME_DIST",       "SYS_AGG06", "NUMBER"        },
+	{ "DENSE_RANK",      "SYS_AGG07", "NUMBER"        },
+	{ "GROUP_ID",        "SYS_AGG08", ""              },
+	{ "GROUPING",        "SYS_AGG09", "-"             },
+	{ "GROUPING_ID",     "SYS_AGG10", "-"             },
+	{ "MAX",             "SYS_AGG11", "NUMBER"        },
+	{ "MIN",             "SYS_AGG12", "NUMBER"        },
+	{ "PERCENTILE_CONT", "SYS_AGG13", "NUMBER"        },
+	{ "PERCENTILE_DISC", "SYS_AGG14", "NUMBER"        },
+	{ "PERCENT_RANK",    "SYS_AGG15", "NUMBER"        },
+	{ "RANK",            "SYS_AGG16", "NUMBER"        },
+	{ "REGR_SLOPE",      "SYS_AGG17", "NUMBER NUMBER" },
+	{ "REGR_INTERCEPT",  "SYS_AGG18", "NUMBER NUMBER" },
+	{ "REGR_COUNT",      "SYS_AGG19", "NUMBER NUMBER" },
+	{ "REGR_R2",         "SYS_AGG20", "NUMBER NUMBER" },
+	{ "REGR_AVGX",       "SYS_AGG21", "NUMBER NUMBER" },
+	{ "REGR_AVGY",       "SYS_AGG22", "NUMBER NUMBER" },
+	{ "REGR_SXX",        "SYS_AGG23", "NUMBER NUMBER" },
+	{ "REGR_SYY",        "SYS_AGG24", "NUMBER NUMBER" },
+	{ "REGR_SXY",        "SYS_AGG25", "NUMBER NUMBER" },
+	{ "STDDEV",          "SYS_AGG26", "NUMBER"        },
+	{ "STDDEV_POP",      "SYS_AGG27", "NUMBER"        },
+	{ "STDDEV_SAMP",     "SYS_AGG28", "NUMBER"        },
+	{ "SUM",             "SYS_AGG29", "NUMBER"        },
+	{ "VAR_POP",         "SYS_AGG30", "NUMBER"        },
+	{ "VAR_SAMP",        "SYS_AGG31", "NUMBER"        },
+	{ "VARIANCE",        "SYS_AGG32", "NUMBER"        }
+};
+static const gda_oracle_aggregate_t
+*aggregate_end = aggregate_tab + (sizeof(aggregate_tab)/sizeof(gda_oracle_aggregate_t));
 
-	value = gda_value_new_string (str);
-	list.data = value;
-	list.next = NULL;
-	list.prev = NULL;
+static GTree *
+build_aggregate_tree (void)
+{
+	GTree *tree;
+	const gda_oracle_aggregate_t *ptr;
 
-	gda_data_model_append_row (GDA_DATA_MODEL (recset), &list);
+	tree = g_tree_new((GCompareFunc)strcmp);
+	for (ptr = aggregate_tab; ptr < aggregate_end; ptr++)
+		g_tree_insert(tree, (gpointer)ptr->name, (gpointer)ptr);
+	return tree;
+}
 
-	gda_value_free (value);
+static gboolean
+aggregate_foreach (gpointer key, gpointer value, gpointer data)
+{
+	gda_oracle_aggregate_t *ptr = value;
+	GList *list = NULL;
+	GList *listp;
+	list = g_list_append(list, gda_value_new_string(ptr->name));
+	list = g_list_append(list, gda_value_new_string(ptr->uid));
+	list = g_list_append(list, gda_value_new_string("SYS"));
+	list = g_list_append(list, gda_value_new_string(""));
+	list = g_list_append(list, gda_value_new_string("NUMBER"));
+	list = g_list_append(list, gda_value_new_string(ptr->args));
+	list = g_list_append(list, gda_value_new_string(""));
+	gda_data_model_append_row (GDA_DATA_MODEL(data), list);
+	for (listp = list; listp; listp = listp->next)
+		gda_value_free(listp->data);
+	g_list_free(list);
+	return FALSE;
 }
 
 static GdaDataModel *
 get_oracle_aggregates (GdaConnection *cnc, GdaParameterList *params)
 {
+	GdaOracleConnectionData *priv_data;
 	GdaDataModelArray *recset;
+	GdaParameter *par = NULL;
+	gchar *name, *uc_name;
+	gda_oracle_aggregate_t *row;
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
+	if (!priv_data) {
+		gda_connection_add_error_string (cnc, _("Invalid Oracle handle"));
+		return NULL;
+	}
 
+	/* if necessary build the GTree of aggregates. */
+	if (priv_data->aggregates == NULL)
+		priv_data->aggregates = build_aggregate_tree();
+	
 	/* create the recordset */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (1);
-	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Name"));
+	recset = (GdaDataModelArray *) gda_data_model_array_new (7);
+	gda_data_model_set_column_title (recset, 0, _("Aggregate"));
+	gda_data_model_set_column_title (recset, 1, _("Id"));
+	gda_data_model_set_column_title (recset, 2, _("Owner"));
+	gda_data_model_set_column_title (recset, 3, _("Comments"));
+	gda_data_model_set_column_title (recset, 4, _("OutType"));
+	gda_data_model_set_column_title (recset, 5, _("InType"));
+	gda_data_model_set_column_title (recset, 6, _("Definition"));
 
-	/* fill the recordset */
-	add_string_row (recset, "abs");
-	add_string_row (recset, "acos");
-	add_string_row (recset, "ascii");
-	add_string_row (recset, "asin");
-	add_string_row (recset, "atan");
-	add_string_row (recset, "atan2");
-	add_string_row (recset, "ceil");
-	add_string_row (recset, "concat");
-	add_string_row (recset, "cos");
-	add_string_row (recset, "cosh");
-	add_string_row (recset, "count");
-	add_string_row (recset, "decode");
-	add_string_row (recset, "exp");
-	add_string_row (recset, "floor");
-	add_string_row (recset, "greatest");
-	add_string_row (recset, "instr");
-	add_string_row (recset, "least");
-	add_string_row (recset, "length");
-	add_string_row (recset, "ln");
-	add_string_row (recset, "log");
-	add_string_row (recset, "lower");
-	add_string_row (recset, "lpad");
-	add_string_row (recset, "ltrim");
-	add_string_row (recset, "max");
-	add_string_row (recset, "min");
-	add_string_row (recset, "mod");
-	add_string_row (recset, "power");
-	add_string_row (recset, "replace");	
-	add_string_row (recset, "reverse");
-	add_string_row (recset, "round");
-	add_string_row (recset, "rpad");
-	add_string_row (recset, "rtrim");
-	add_string_row (recset, "sign");
-	add_string_row (recset, "sinh");
-	add_string_row (recset, "sqrt");
-	add_string_row (recset, "substr");
-	add_string_row (recset, "sysdate");
-	add_string_row (recset, "tanh");
-	add_string_row (recset, "to_char");
-	add_string_row (recset, "to_date");
-	add_string_row (recset, "trim");
-	add_string_row (recset, "trunc");
-	add_string_row (recset, "upper");
-	add_string_row (recset, "user");
+	if (params)
+		par = gda_parameter_list_find (params, "name");
+	if (par) {
+		name = (gchar *)gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
+		uc_name = g_ascii_strup(name, -1);
+		if ((row = g_tree_lookup(priv_data->aggregates, uc_name)))
+			aggregate_foreach(uc_name, row, recset);
+		g_free(uc_name);
+		g_free(name);
+	} else
+		g_tree_foreach(priv_data->aggregates, aggregate_foreach, recset);
 
 	return GDA_DATA_MODEL (recset);
 }
@@ -1137,8 +1193,8 @@ gda_oracle_table_tree(GdaConnection *cnc)
 	return NULL;
     recset = reclist->data;
     priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
-    tree = g_tree_new(strcmp);
-    for (i = 0; (row = gda_data_model_get_row(recset, i)); i++) {
+    tree = g_tree_new((GCompareFunc)strcmp);
+    for (i = 0; (row = (GdaRow *)gda_data_model_get_row(recset, i)); i++) {
 	value = gda_row_get_value(row, 0);
 	name = gda_value_stringify (value);
 	value = gda_row_get_value(row, 1);
@@ -1173,8 +1229,8 @@ gda_oracle_view_tree(GdaConnection *cnc)
 	return NULL;
     recset = reclist->data;
     priv_data = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_ORACLE_HANDLE);
-    tree = g_tree_new(strcmp);
-    for (i = 0; (row = gda_data_model_get_row(recset, i)); i++) {
+    tree = g_tree_new((GCompareFunc)strcmp);
+    for (i = 0; (row = (GdaRow *)gda_data_model_get_row(recset, i)); i++) {
 	value = gda_row_get_value(row, 0);
 	name = gda_value_stringify (value);
 	g_tree_insert(tree, name, priv_data->schema);
@@ -1219,8 +1275,9 @@ g_string_right_trim (GString * input)
 }
 
 static GHashTable *
-get_oracle_index_data (GdaConnection *cnc, const gchar *tblname)
+get_oracle_index_data (GdaConnection *cnc, const gchar *owner, const gchar *tblname)
 {
+	gchar *sql;
 	GList *reclist;
 	GdaDataModel *recset;
 	GdaOracleIndexData *index_data;
@@ -1233,31 +1290,34 @@ get_oracle_index_data (GdaConnection *cnc, const gchar *tblname)
 	GdaValue *value;
 	gint i;
 
-
-	gchar *sql = g_strdup ("SELECT substr (c1.column_name,1,30) column_name, "
-				"con.constraint_type constraint_type, "
-				"'' uniqueness, "
-				"c2.table_name reference "
-				"FROM user_cons_columns c1, user_cons_columns c2, user_constraints con "
-				"WHERE con.constraint_name = c1.constraint_name " 
-				"AND con.constraint_type = 'R' "
-				"AND con.constraint_type IS NOT NULL "
-				"AND c1.table_name = upper (:TBLNAME) "
-				"AND con.r_constraint_name = c2.constraint_name "
-				"UNION "
-				"SELECT substr (c.column_name,1,30) column_name, "
-				"t.constraint_type constraint_type, "
-				"substr (i.uniqueness, 1, 6) uniqueness, "
-				"'' reference "
-				"FROM user_indexes i, user_ind_columns c, user_constraints t "
-				"WHERE i.table_name = upper (:TBLNAME) "
-				"AND t.constraint_type != 'R' "
-				"AND i.index_name = c.index_name "
-				"AND i.index_name = t.constraint_name "
-				"ORDER BY 1" );
-
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 
+	sql = g_strdup ("SELECT c1.column_name column_name, "
+			"con.constraint_type constraint_type, "
+			"'' uniqueness, "
+			"c2.table_name reference "
+			"FROM all_cons_columns c1, all_cons_columns c2, all_constraints con "
+			"WHERE con.constraint_name = c1.constraint_name " 
+			"AND con.constraint_type = 'R' "
+			"AND con.constraint_type IS NOT NULL "
+			"AND c1.table_name = upper (:TBLNAME) "
+			"AND c1.owner = upper (:OWNER) "
+			"AND con.r_constraint_name = c2.constraint_name "
+			"UNION ALL "
+			"SELECT c.column_name column_name, "
+			"t.constraint_type constraint_type, "
+			"substr (i.uniqueness, 1, 6) uniqueness, "
+			"'' reference "
+			"FROM all_indexes i, all_ind_columns c, all_constraints t "
+			"WHERE i.table_name = upper (:TBLNAME) "
+			"AND i.table_owner = upper (:OWNER) "
+			"AND t.constraint_type != 'R' "
+			"AND i.index_name = c.index_name "
+			"AND i.index_name = t.constraint_name "
+			"ORDER BY 1" );
+
+	gda_parameter_list_add_parameter (query_params, 
+			gda_parameter_new_string (":OWNER", owner));
 	gda_parameter_list_add_parameter (query_params, 
 			gda_parameter_new_string (":TBLNAME", tblname));
 
@@ -1277,13 +1337,13 @@ get_oracle_index_data (GdaConnection *cnc, const gchar *tblname)
 	index_data = g_new0 (GdaOracleIndexData, 1);
 	references = g_string_new ("");
 
-	value = gda_data_model_get_value_at (recset, 0, 0);
+	value = (GdaValue *)gda_data_model_get_value_at (recset, 0, 0);
 	colname = g_string_right_trim (g_string_new (gda_value_get_string (value)));
 
 	for (i = 0; i < nrows; i += 1) {
 		gchar *constraint_type = "";
 
-		value = gda_data_model_get_value_at (recset, 1, i);
+		value = (GdaValue *)gda_data_model_get_value_at (recset, 1, i);
 		if (!value)
 			continue;
 		if (gda_value_is_null (value))
@@ -1294,14 +1354,14 @@ get_oracle_index_data (GdaConnection *cnc, const gchar *tblname)
 		if (!strcmp (constraint_type, "P"))
 			index_data->primary_key = TRUE;
 
-		value = gda_data_model_get_value_at (recset, 2, i);
+		value = (GdaValue *)gda_data_model_get_value_at (recset, 2, i);
 		if (value) 
 			if (!gda_value_is_null (value))
 				if (!strcmp (gda_value_get_string (value), "UNIQUE"))  
 					index_data->unique = TRUE;
 
 		if (!strcmp (constraint_type, "R")) {
-			value = gda_data_model_get_value_at (recset, 3, i);
+			value = (GdaValue *)gda_data_model_get_value_at (recset, 3, i);
 			if (value)
 				if (!gda_value_is_null (value)) {
 					if (references->len > 0) 
@@ -1316,7 +1376,7 @@ get_oracle_index_data (GdaConnection *cnc, const gchar *tblname)
 			break;
 		}
 
-		value = gda_data_model_get_value_at (recset, 0, i+1);
+		value = (GdaValue *)gda_data_model_get_value_at (recset, 0, i+1);
 		newcolname = g_string_right_trim (g_string_new (gda_value_get_string (value)));
 
 		if (strcmp (colname->str, newcolname->str)) {
@@ -1393,13 +1453,14 @@ gda_oracle_fill_md_data (const gchar *tblname,
 	/* look up the table name to get the fully qualified name. */
 	upc_tblname = g_ascii_strup(tblname, -1);
 	if ((owner = g_tree_lookup(priv_data->tables, upc_tblname)) == NULL)
-	    owner = g_tree_lookup(priv_data->views, upc_tblname);
+		owner = g_tree_lookup(priv_data->views, upc_tblname);
 	g_free(upc_tblname);
-	if (owner == NULL)
-	    fq_tblname = tblname;
+	if (owner == NULL) {
+		fq_tblname = (gchar *)tblname;
+		owner = priv_data->schema;
+	}
 	else
-	    fq_tblname = g_strjoin(".", owner, tblname, NULL);
-	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "full name is '%s'",fq_tblname);
+		fq_tblname = g_strjoin(".", owner, tblname, NULL);
 	
 	/* Describe the table */
 	result = OCIDescribeAny (priv_data->hservice,
@@ -1565,7 +1626,7 @@ gda_oracle_fill_md_data (const gchar *tblname,
 		return NULL;
 	}
 
-	h_table_index = get_oracle_index_data (cnc, fq_tblname);
+	h_table_index = get_oracle_index_data (cnc, owner, tblname);
 
 	for (i = 1; i <= numcols; i += 1) {
 		text *strp;
@@ -1573,7 +1634,7 @@ gda_oracle_fill_md_data (const gchar *tblname,
 		ub2 type;
 		gchar *colname;
 		gchar *typename;
-		ub2 nullable;
+		ub1 nullable;
 		ub2 defined_size;
 		ub2 scale;
 
@@ -1681,7 +1742,7 @@ gda_oracle_fill_md_data (const gchar *tblname,
 					(ub4) OCI_DTYPE_PARAM,
 					(dvoid *) &nullable,
 					(ub4 *) 0,
-					(ub4) OCI_ATTR_DATA_SIZE,
+					(ub4) OCI_ATTR_IS_NULL,
 					(OCIError *) priv_data->herr);
 		if (!gda_oracle_check_result (result, cnc, priv_data, OCI_HTYPE_ERROR,
 				_("Could not get the Oracle field nullable attribute"))) {
@@ -1816,7 +1877,6 @@ get_oracle_tables (GdaConnection *cnc, GdaParameterList *params)
 	par = gda_parameter_list_find (params, "namespace");
     if (par)
     {
-	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "parameterised get_tables");
 	namespace = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
 	upc_namespace = g_ascii_strup(namespace, -1);
 
@@ -1832,7 +1892,7 @@ get_oracle_tables (GdaConnection *cnc, GdaParameterList *params)
 
 	reclist = process_sql_commands (NULL, cnc, sql, NULL, 
 					GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-	g_free(upc_namespace);
+	g_free((gpointer)upc_namespace);
 	g_free (sql);
 
 	if (!reclist)
@@ -1849,7 +1909,6 @@ get_oracle_tables (GdaConnection *cnc, GdaParameterList *params)
 		{ N_("Comments"), GDA_VALUE_TYPE_STRING },
 		{ N_("SQL"),      GDA_VALUE_TYPE_STRING }
 	};
-	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "default get_tables");
 	priv_data = g_object_get_data(G_OBJECT(cnc),OBJECT_DATA_ORACLE_HANDLE);
 	if (priv_data->tables == NULL)
 	    if (gda_oracle_table_tree(cnc) == NULL)
@@ -1860,7 +1919,6 @@ get_oracle_tables (GdaConnection *cnc, GdaParameterList *params)
 
 	g_tree_foreach(priv_data->tables, get_tables_foreach, recset);
     }
-    g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "get_tables returning %p", recset);
     return recset;
 }
 
@@ -1881,7 +1939,6 @@ get_oracle_views (GdaConnection *cnc, GdaParameterList *params)
 	par = gda_parameter_list_find (params, "namespace");
     if (par)
     {
-	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "parameterised get_views");
 	namespace = gda_value_get_string ((GdaValue *) gda_parameter_get_value (par));
 	upc_namespace = g_ascii_strup(namespace, -1);
 
@@ -1897,7 +1954,7 @@ get_oracle_views (GdaConnection *cnc, GdaParameterList *params)
 
 	reclist = process_sql_commands (NULL, cnc, sql, NULL, 
 					GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-	g_free(upc_namespace);
+	g_free((gpointer)upc_namespace);
 	g_free (sql);
 
 	if (!reclist)
@@ -1914,7 +1971,6 @@ get_oracle_views (GdaConnection *cnc, GdaParameterList *params)
 		{ N_("Comments"), GDA_VALUE_TYPE_STRING },
 		{ N_("SQL"),      GDA_VALUE_TYPE_STRING }
 	};
-	g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "default get_views");
 	priv_data = g_object_get_data(G_OBJECT(cnc),OBJECT_DATA_ORACLE_HANDLE);
 	if (priv_data->views == NULL)
 	    if (gda_oracle_view_tree(cnc) == NULL)
@@ -1925,7 +1981,6 @@ get_oracle_views (GdaConnection *cnc, GdaParameterList *params)
 	
 	g_tree_foreach(priv_data->views, get_tables_foreach, recset);
     }
-    g_log("gda-oracle", G_LOG_LEVEL_DEBUG, "get_views returning %p", recset);
 	return recset;
 }
 
@@ -1943,7 +1998,7 @@ get_oracle_views (GdaConnection *cnc, GdaParameterList *params)
  * see any use for any user passed parameters at this point.
  */
 static GdaDataModel *
-get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionSchema schema, gint nargs)
+get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionSchema schema, gint nargs, const gchar *title)
 {
 	GList *reclist;
 	GdaDataModel *recset;
@@ -1994,43 +2049,63 @@ get_oracle_objects (GdaConnection *cnc, GdaParameterList *params, GdaConnectionS
 	if (!reclist)
 		return NULL;
 
-	recset = GDA_DATA_MODEL (reclist->data);
+	if ((recset = GDA_DATA_MODEL (reclist->data)) != NULL)
+		gda_data_model_set_column_title (recset, 0, title);
 	g_list_free (reclist);
-
+	
 	return recset;
 }
 
+typedef struct
+{
+    const gchar  *name;
+    GdaValueType type;
+} ora_native_type;
+
+static const ora_native_type ora_type_tab[] =
+{
+	{ "BFILE",     GDA_VALUE_TYPE_BINARY    },
+	{ "BLOB",      GDA_VALUE_TYPE_BINARY    },
+	{ "CHAR",      GDA_VALUE_TYPE_STRING    },
+	{ "CLOB",      GDA_VALUE_TYPE_STRING    },
+	{ "DATE",      GDA_VALUE_TYPE_TIMESTAMP },
+	{ "NUMBER",    GDA_VALUE_TYPE_NUMERIC   },
+	{ "LONG",      GDA_VALUE_TYPE_STRING    },
+	{ "LONG RAW",  GDA_VALUE_TYPE_BINARY    },
+	{ "RAW",       GDA_VALUE_TYPE_BINARY    },
+	{ "ROWID",     GDA_VALUE_TYPE_STRING    },
+	{ "TIMESTAMP", GDA_VALUE_TYPE_TIMESTAMP },
+	{ "VARCHAR2",  GDA_VALUE_TYPE_STRING    }
+};
+static const ora_native_type *
+ora_type_end = ora_type_tab+sizeof(ora_type_tab)/sizeof(ora_native_type);
+
 static GdaDataModel *
 get_oracle_types (GdaConnection *cnc, GdaParameterList *params)
 {
-	GdaDataModelArray *recset;
+    GdaDataModelArray *recset;
+    ora_native_type   *otp;
+    GList             *value_list;
+    
+    /* create the recordset */
+    recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (4));
+    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
+    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 1, _("Owner"));
+    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 2, _("Comments"));    gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 3, _("GDA type"));
+    /* fill the recordset */
+    for (otp = ora_type_tab; otp < ora_type_end; otp++)
+    {
+        value_list = NULL;
+        value_list = g_list_append (value_list, gda_value_new_string (otp->name));
+        value_list = g_list_append (value_list, gda_value_new_string ("SYS"));
+        value_list = g_list_append (value_list, gda_value_new_string ("NULL"));
+        value_list = g_list_append (value_list, gda_value_new_type (otp->type));
 
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
-
-	/* create the recordset */
-	recset = (GdaDataModelArray *) gda_data_model_array_new (1);
-	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Type"));
-
-	/* fill the recordset */
-	add_string_row (recset, "blob");
-	add_string_row (recset, "date");
-	add_string_row (recset, "datetime");
-	add_string_row (recset, "decimal");
-	add_string_row (recset, "double");
-	add_string_row (recset, "enum");
-	add_string_row (recset, "float");
-	add_string_row (recset, "int24");
-	add_string_row (recset, "long");
-	add_string_row (recset, "longlong");
-	add_string_row (recset, "set");
-	add_string_row (recset, "short");
-	add_string_row (recset, "string");
-	add_string_row (recset, "time");
-	add_string_row (recset, "timestamp");
-	add_string_row (recset, "tiny");
-	add_string_row (recset, "year");
-
-	return GDA_DATA_MODEL (recset);
+	gda_data_model_append_row (GDA_DATA_MODEL (recset), value_list);
+        g_list_foreach (value_list, (GFunc) gda_value_free, NULL);
+        g_list_free (value_list);
+    }
+    return GDA_DATA_MODEL (recset);
 }
 
 static void 
@@ -2096,23 +2171,19 @@ gda_oracle_provider_get_schema (GdaServerProvider *provider,
 		recset = get_oracle_databases (cnc, params);
 		break;
 	case GDA_CONNECTION_SCHEMA_INDEXES :
-		recset = get_oracle_objects (cnc, params, schema, 1);
-		gda_data_model_set_column_title (recset, 0, _("Indexes"));
+		recset = get_oracle_objects (cnc, params, schema, 1, _("Indexes"));
 		break;
 	case GDA_CONNECTION_SCHEMA_PROCEDURES :
-		recset = get_oracle_objects (cnc, params, schema, 8);
-		gda_data_model_set_column_title (recset, 0, _("Procedures"));
+		recset = get_oracle_objects (cnc, params, schema, 8, _("Procedures"));
 		break;
 	case GDA_CONNECTION_SCHEMA_SEQUENCES :
-		recset = get_oracle_objects (cnc, params, schema, 4);
-		gda_data_model_set_column_title (recset, 0, _("Sequences"));
+		recset = get_oracle_objects (cnc, params, schema, 4, _("Sequences"));
 		break;
 	case GDA_CONNECTION_SCHEMA_TABLES :
 		recset = get_oracle_tables (cnc, params);
 		break;
 	case GDA_CONNECTION_SCHEMA_TRIGGERS :
-		recset = get_oracle_objects (cnc, params, schema, 1);
-		gda_data_model_set_column_title (recset, 0, _("Triggers"));
+		recset = get_oracle_objects (cnc, params, schema, 1, _("Triggers"));
 		break;
 	case GDA_CONNECTION_SCHEMA_VIEWS :
 		recset = get_oracle_views(cnc, params);
@@ -2121,6 +2192,12 @@ gda_oracle_provider_get_schema (GdaServerProvider *provider,
 	default :
 		recset = NULL;
 	}
-
 	return recset;
 }
+
+/*
+  Local Variables:
+  mode:C
+  c-basic-offset: 8
+  End:
+*/
